@@ -151,17 +151,14 @@ class QTestQmlModule : public QQmlExtensionPlugin
 
 public:
     QTestQmlModule(QObject *parent = 0) : QQmlExtensionPlugin(parent) { initResources(); }
-    virtual void registerTypes(const char *uri)
+    void registerTypes(const char *uri) Q_DECL_OVERRIDE
     {
         Q_ASSERT(QLatin1String(uri) == QLatin1String("QtTest"));
         qmlRegisterType<QuickTestResult, 0>(uri,1,0,"TestResult");
         qmlRegisterType<QuickTestResult, 1>(uri,1,1,"TestResult");
         qmlRegisterType<QuickTestEvent>(uri,1,0,"TestEvent");
         qmlRegisterType<QuickTestUtil>(uri,1,0,"TestUtil");
-    }
-
-    void initializeEngine(QQmlEngine *, const char *)
-    {
+        qmlRegisterType<QQuickTouchEventSequence>();
     }
 };
 
