@@ -47,6 +47,7 @@
 #include <private/qv4ssa_p.h>
 
 #include "qqmlpropertycachecreator_p.h"
+#include "qv4jssimplifier_p.h"
 
 #define COMPILE_EXCEPTION(token, desc) \
     { \
@@ -57,12 +58,13 @@
 QT_BEGIN_NAMESPACE
 
 QQmlTypeCompiler::QQmlTypeCompiler(QQmlEnginePrivate *engine, QQmlTypeData *typeData,
-                                   QmlIR::Document *parsedQML, const QQmlRefPointer<QQmlTypeNameCache> &importCache,
-                                   const QV4::CompiledData::ResolvedTypeReferenceMap &resolvedTypeCache)
+                                   QmlIR::Document *parsedQML, const QQmlRefPointer<QQmlTypeNameCache> &typeNameCache,
+                                   const QV4::CompiledData::ResolvedTypeReferenceMap &resolvedTypeCache, const QV4::CompiledData::DependentTypesHasher &dependencyHasher)
     : resolvedTypes(resolvedTypeCache)
     , engine(engine)
     , typeData(typeData)
-    , importCache(importCache)
+    , dependencyHasher(dependencyHasher)
+    , typeNameCache(typeNameCache)
     , document(parsedQML)
 {
 }
@@ -138,12 +140,12 @@ QV4::CompiledData::CompilationUnit *QQmlTypeCompiler::compile()
             sss.scan();
         }
 
-        QmlIR::JSCodeGen v4CodeGenerator(typeData->finalUrlString(), document->code, &document->jsModule, &document->jsParserEngine, document->program, importCache, &document->jsGenerator.stringTable);
+        QmlIR::JSCodeGen v4CodeGenerator(typeData->finalUrlString(), document->code, &document->jsModule, &document->jsParserEngine, document->program, typeNameCache, &document->jsGenerator.stringTable);
         QQmlJSCodeGenerator jsCodeGen(this, &v4CodeGenerator);
         if (!jsCodeGen.generateCodeForComponents())
             return nullptr;
 
-        QQmlJavaScriptBindingExpressionSimplificationPass pass(this);
+        QQmlJavaScriptBindingExpressionSimplificationPass pass(document->objects, &document->jsModule, &document->jsGenerator);
         pass.reduceTranslationBindings();
 
         QV4::ExecutionEngine *v4 = engine->v4engine();
@@ -156,7 +158,7 @@ QV4::CompiledData::CompilationUnit *QQmlTypeCompiler::compile()
     // Generate QML compiled type data structures
 
     QmlIR::QmlUnitGenerator qmlGenerator;
-    QV4::CompiledData::Unit *qmlUnit = qmlGenerator.generate(*document, QQmlEnginePrivate::get(engine), resolvedTypes);
+    QV4::CompiledData::Unit *qmlUnit = qmlGenerator.generate(*document, dependencyHasher);
 
     Q_ASSERT(document->javaScriptCompilationUnit);
     // The js unit owns the data and will free the qml unit.
@@ -164,7 +166,7 @@ QV4::CompiledData::CompilationUnit *QQmlTypeCompiler::compile()
 
     QV4::CompiledData::CompilationUnit *compilationUnit = document->javaScriptCompilationUnit;
     compilationUnit = document->javaScriptCompilationUnit;
-    compilationUnit->importCache = importCache;
+    compilationUnit->typeNameCache = typeNameCache;
     compilationUnit->resolvedTypes = resolvedTypes;
     compilationUnit->propertyCaches = std::move(m_propertyCaches);
     Q_ASSERT(compilationUnit->propertyCaches.count() == static_cast<int>(compilationUnit->data->nObjects));
@@ -471,7 +473,7 @@ bool SignalHandlerConverter::convertSignalHandlerExpressionsToFunctionDeclaratio
         QQmlJS::MemoryPool *pool = compiler->memoryPool();
 
         QQmlJS::AST::FormalParameterList *paramList = 0;
-        foreach (const QString &param, parameters) {
+        for (const QString &param : qAsConst(parameters)) {
             QStringRef paramNameRef = compiler->newStringRef(param);
 
             if (paramList)
@@ -1108,10 +1110,9 @@ QQmlComponentAndAliasResolver::AliasResolutionResult QQmlComponentAndAliasResolv
                         continue;
                     }
 
-                    // Try again later and resolve the target alias first.
-                    _objectsWithAliases.append(objectIndex);
                     // restore
                     alias->idIndex = idIndex;
+                    // Try again later and resolve the target alias first.
                     break;
                 }
             }
@@ -1348,9 +1349,9 @@ bool QQmlJSCodeGenerator::compileJavaScriptCodeInObjectsRecursively(int objectIn
             functionsToCompile << *foe;
         }
         const QVector<int> runtimeFunctionIndices = v4CodeGen->generateJSCodeForFunctionsAndBindings(functionsToCompile);
-        QList<QQmlError> jsErrors = v4CodeGen->qmlErrors();
+        const QList<QQmlError> jsErrors = v4CodeGen->qmlErrors();
         if (!jsErrors.isEmpty()) {
-            foreach (const QQmlError &e, jsErrors)
+            for (const QQmlError &e : jsErrors)
                 compiler->recordError(e);
             return false;
         }
@@ -1429,348 +1430,4 @@ void QQmlDefaultPropertyMerger::mergeDefaultProperties(int objectIndex)
     }
 }
 
-QQmlJavaScriptBindingExpressionSimplificationPass::QQmlJavaScriptBindingExpressionSimplificationPass(QQmlTypeCompiler *typeCompiler)
-    : QQmlCompilePass(typeCompiler)
-    , qmlObjects(*typeCompiler->qmlObjects())
-    , jsModule(typeCompiler->jsIRModule())
-{
-
-}
-
-void QQmlJavaScriptBindingExpressionSimplificationPass::reduceTranslationBindings()
-{
-    for (int i = 0; i < qmlObjects.count(); ++i)
-        reduceTranslationBindings(i);
-    if (!irFunctionsToRemove.isEmpty()) {
-        QQmlIRFunctionCleanser cleanser(compiler, irFunctionsToRemove);
-        cleanser.clean();
-    }
-}
-
-void QQmlJavaScriptBindingExpressionSimplificationPass::reduceTranslationBindings(int objectIndex)
-{
-    const QmlIR::Object *obj = qmlObjects.at(objectIndex);
-
-    for (QmlIR::Binding *binding = obj->firstBinding(); binding; binding = binding->next) {
-        if (binding->type != QV4::CompiledData::Binding::Type_Script)
-            continue;
-
-        const int irFunctionIndex = obj->runtimeFunctionIndices.at(binding->value.compiledScriptIndex);
-        QV4::IR::Function *irFunction = jsModule->functions.at(irFunctionIndex);
-        if (simplifyBinding(irFunction, binding)) {
-            irFunctionsToRemove.append(irFunctionIndex);
-            jsModule->functions[irFunctionIndex] = 0;
-            delete irFunction;
-        }
-    }
-}
-
-void QQmlJavaScriptBindingExpressionSimplificationPass::visitMove(QV4::IR::Move *move)
-{
-    QV4::IR::Temp *target = move->target->asTemp();
-    if (!target || target->kind != QV4::IR::Temp::VirtualRegister) {
-        discard();
-        return;
-    }
-
-    if (QV4::IR::Call *call = move->source->asCall()) {
-        if (QV4::IR::Name *n = call->base->asName()) {
-            if (n->builtin == QV4::IR::Name::builtin_invalid) {
-                visitFunctionCall(n->id, call->args, target);
-                return;
-            }
-        }
-        discard();
-        return;
-    }
-
-    if (QV4::IR::Name *n = move->source->asName()) {
-        if (n->builtin == QV4::IR::Name::builtin_qml_context
-            || n->builtin == QV4::IR::Name::builtin_qml_imported_scripts_object) {
-            // these are free of side-effects
-            return;
-        }
-        discard();
-        return;
-    }
-
-    if (!move->source->asTemp() && !move->source->asString() && !move->source->asConst()) {
-        discard();
-        return;
-    }
-
-    _temps[target->index] = move->source;
-}
-
-void QQmlJavaScriptBindingExpressionSimplificationPass::visitFunctionCall(const QString *name, QV4::IR::ExprList *args, QV4::IR::Temp *target)
-{
-    // more than one function call?
-    if (_nameOfFunctionCalled) {
-        discard();
-        return;
-    }
-
-    _nameOfFunctionCalled = name;
-
-    _functionParameters.clear();
-    while (args) {
-        int slot;
-        if (QV4::IR::Temp *param = args->expr->asTemp()) {
-            if (param->kind != QV4::IR::Temp::VirtualRegister) {
-                discard();
-                return;
-            }
-            slot = param->index;
-            _functionParameters.append(slot);
-        } else if (QV4::IR::Const *param = args->expr->asConst()) {
-            slot = --_synthesizedConsts;
-            Q_ASSERT(!_temps.contains(slot));
-            _temps[slot] = param;
-            _functionParameters.append(slot);
-        }
-        args = args->next;
-    }
-
-    _functionCallReturnValue = target->index;
-}
-
-void QQmlJavaScriptBindingExpressionSimplificationPass::visitRet(QV4::IR::Ret *ret)
-{
-    // nothing initialized earlier?
-    if (_returnValueOfBindingExpression != -1) {
-        discard();
-        return;
-    }
-    QV4::IR::Temp *target = ret->expr->asTemp();
-    if (!target || target->kind != QV4::IR::Temp::VirtualRegister) {
-        discard();
-        return;
-    }
-    _returnValueOfBindingExpression = target->index;
-}
-
-bool QQmlJavaScriptBindingExpressionSimplificationPass::simplifyBinding(QV4::IR::Function *function, QmlIR::Binding *binding)
-{
-    _canSimplify = true;
-    _nameOfFunctionCalled = 0;
-    _functionParameters.clear();
-    _functionCallReturnValue = -1;
-    _temps.clear();
-    _returnValueOfBindingExpression = -1;
-    _synthesizedConsts = 0;
-
-    // It would seem unlikely that function with some many basic blocks (after optimization)
-    // consists merely of a qsTr call or a constant value return ;-)
-    if (function->basicBlockCount() > 10)
-        return false;
-
-    for (QV4::IR::BasicBlock *bb : function->basicBlocks()) {
-        for (QV4::IR::Stmt *s : bb->statements()) {
-            visit(s);
-            if (!_canSimplify)
-                return false;
-        }
-        if (!_canSimplify)
-            return false;
-    }
-
-    if (_returnValueOfBindingExpression == -1)
-        return false;
-
-    if (_canSimplify) {
-        if (_nameOfFunctionCalled) {
-            if (_functionCallReturnValue != _returnValueOfBindingExpression)
-                return false;
-            return detectTranslationCallAndConvertBinding(binding);
-        }
-    }
-
-    return false;
-}
-
-bool QQmlJavaScriptBindingExpressionSimplificationPass::detectTranslationCallAndConvertBinding(QmlIR::Binding *binding)
-{
-    if (*_nameOfFunctionCalled == QLatin1String("qsTr")) {
-        QString translation;
-        QV4::CompiledData::TranslationData translationData;
-        translationData.number = -1;
-        translationData.commentIndex = 0; // empty string
-
-        QVector<int>::ConstIterator param = _functionParameters.constBegin();
-        QVector<int>::ConstIterator end = _functionParameters.constEnd();
-        if (param == end)
-            return false;
-
-        QV4::IR::String *stringParam = _temps[*param]->asString();
-        if (!stringParam)
-            return false;
-
-        translation = *stringParam->value;
-
-        ++param;
-        if (param != end) {
-            stringParam = _temps[*param]->asString();
-            if (!stringParam)
-                return false;
-            translationData.commentIndex = compiler->registerString(*stringParam->value);
-            ++param;
-
-            if (param != end) {
-                QV4::IR::Const *constParam = _temps[*param]->asConst();
-                if (!constParam || constParam->type != QV4::IR::SInt32Type)
-                    return false;
-
-                translationData.number = int(constParam->value);
-                ++param;
-            }
-        }
-
-        if (param != end)
-            return false;
-
-        binding->type = QV4::CompiledData::Binding::Type_Translation;
-        binding->stringIndex = compiler->registerString(translation);
-        binding->value.translationData = translationData;
-        return true;
-    } else if (*_nameOfFunctionCalled == QLatin1String("qsTrId")) {
-        QString id;
-        QV4::CompiledData::TranslationData translationData;
-        translationData.number = -1;
-        translationData.commentIndex = 0; // empty string, but unused
-
-        QVector<int>::ConstIterator param = _functionParameters.constBegin();
-        QVector<int>::ConstIterator end = _functionParameters.constEnd();
-        if (param == end)
-            return false;
-
-        QV4::IR::String *stringParam = _temps[*param]->asString();
-        if (!stringParam)
-            return false;
-
-        id = *stringParam->value;
-
-        ++param;
-        if (param != end) {
-            QV4::IR::Const *constParam = _temps[*param]->asConst();
-            if (!constParam || constParam->type != QV4::IR::SInt32Type)
-                return false;
-
-            translationData.number = int(constParam->value);
-            ++param;
-        }
-
-        if (param != end)
-            return false;
-
-        binding->type = QV4::CompiledData::Binding::Type_TranslationById;
-        binding->stringIndex = compiler->registerString(id);
-        binding->value.translationData = translationData;
-        return true;
-    } else if (*_nameOfFunctionCalled == QLatin1String("QT_TR_NOOP") || *_nameOfFunctionCalled == QLatin1String("QT_TRID_NOOP")) {
-        QVector<int>::ConstIterator param = _functionParameters.constBegin();
-        QVector<int>::ConstIterator end = _functionParameters.constEnd();
-        if (param == end)
-            return false;
-
-        QV4::IR::String *stringParam = _temps[*param]->asString();
-        if (!stringParam)
-            return false;
-
-        ++param;
-        if (param != end)
-            return false;
-
-        binding->type = QV4::CompiledData::Binding::Type_String;
-        binding->stringIndex = compiler->registerString(*stringParam->value);
-        return true;
-    } else if (*_nameOfFunctionCalled == QLatin1String("QT_TRANSLATE_NOOP")) {
-        QVector<int>::ConstIterator param = _functionParameters.constBegin();
-        QVector<int>::ConstIterator end = _functionParameters.constEnd();
-        if (param == end)
-            return false;
-
-        ++param;
-        if (param == end)
-            return false;
-
-        QV4::IR::String *stringParam = _temps[*param]->asString();
-        if (!stringParam)
-            return false;
-
-        ++param;
-        if (param != end)
-            return false;
-
-        binding->type = QV4::CompiledData::Binding::Type_String;
-        binding->stringIndex = compiler->registerString(*stringParam->value);
-        return true;
-    }
-    return false;
-}
-
-QQmlIRFunctionCleanser::QQmlIRFunctionCleanser(QQmlTypeCompiler *typeCompiler, const QVector<int> &functionsToRemove)
-    : QQmlCompilePass(typeCompiler)
-    , module(typeCompiler->jsIRModule())
-    , functionsToRemove(functionsToRemove)
-{
-}
-
-void QQmlIRFunctionCleanser::clean()
-{
-    QVector<QV4::IR::Function*> newFunctions;
-    newFunctions.reserve(module->functions.count() - functionsToRemove.count());
-
-    newFunctionIndices.resize(module->functions.count());
-
-    for (int i = 0; i < module->functions.count(); ++i) {
-        QV4::IR::Function *f = module->functions.at(i);
-        Q_ASSERT(f || functionsToRemove.contains(i));
-        if (f) {
-            newFunctionIndices[i] = newFunctions.count();
-            newFunctions << f;
-        }
-    }
-
-    module->functions = newFunctions;
-
-    foreach (QV4::IR::Function *function, module->functions) {
-        for (QV4::IR::BasicBlock *block : function->basicBlocks()) {
-            for (QV4::IR::Stmt *s : block->statements()) {
-                visit(s);
-            }
-        }
-    }
-
-    foreach (QmlIR::Object *obj, *compiler->qmlObjects()) {
-        for (int i = 0; i < obj->runtimeFunctionIndices.count; ++i)
-            obj->runtimeFunctionIndices[i] = newFunctionIndices[obj->runtimeFunctionIndices.at(i)];
-    }
-}
-
-void QQmlIRFunctionCleanser::visit(QV4::IR::Stmt *s)
-{
-
-    switch (s->stmtKind) {
-    case QV4::IR::Stmt::PhiStmt:
-        // nothing to do
-        break;
-    default:
-        STMT_VISIT_ALL_KINDS(s);
-        break;
-    }
-}
-
-void QQmlIRFunctionCleanser::visit(QV4::IR::Expr *e)
-{
-    switch (e->exprKind) {
-    case QV4::IR::Expr::ClosureExpr: {
-        auto closure = e->asClosure();
-        closure->value = newFunctionIndices.at(closure->value);
-    } break;
-    default:
-        EXPR_VISIT_ALL_KINDS(e);
-        break;
-    }
-}
-
 QT_END_NAMESPACE
