@@ -348,11 +348,7 @@ Module::~Module()
 
 void Module::setFileName(const QString &name)
 {
-    if (fileName.isEmpty())
-        fileName = name;
-    else {
-        Q_ASSERT(fileName == name);
-    }
+    fileName = name;
 }
 
 Function::Function(Module *module, Function *outer, const QString &name)
@@ -448,10 +444,6 @@ void Function::setScheduledBlocks(const QVector<BasicBlock *> &scheduled)
     Q_ASSERT(!_allBasicBlocks);
     _allBasicBlocks = new QVector<BasicBlock *>(basicBlocks());
     _basicBlocks = scheduled;
-}
-
-void Function::renumberBasicBlocks()
-{
     for (int i = 0, ei = basicBlockCount(); i != ei; ++i)
         basicBlock(i)->changeIndex(i);
 }
