@@ -266,7 +266,7 @@ int quick_test_main(int argc, char **argv, const char *name, const char *sourceD
 
     QuickTestResult::parseArgs(testArgC, testArgV.data());
 
-#ifndef QT_NO_TRANSLATION
+#if QT_CONFIG(translation)
     QTranslator translator;
     if (!translationFile.isEmpty()) {
         if (translator.load(translationFile)) {
@@ -373,9 +373,6 @@ int quick_test_main(int argc, char **argv, const char *name, const char *sourceD
             // and then wait for quit indication.
             view->setFramePosition(QPoint(50, 50));
             if (view->size().isEmpty()) { // Avoid hangs with empty windows.
-                qWarning().nospace()
-                    << "Test '" << QDir::toNativeSeparators(path) << "' has invalid size "
-                    << view->size() << ", resizing.";
                 view->resize(200, 200);
             }
             view->show();
